//
// Copyright (c) 2001-2020 by PDFTron Systems Inc. All Rights Reserved.
//

using System;
using pdftron;
using pdftron.Common;
using pdftron.Filters;
using pdftron.SDF;
using pdftron.PDF;
using System.Collections;
using System.Collections.Generic;
using System.Security.Policy;

namespace ElementReaderTestCS
{
	class Class1
	{
		private static pdftron.PDFNetLoader pdfNetLoader = pdftron.PDFNetLoader.Instance();
		static Class1() {}

		class MarkedContentItem
        {
			public MarkedContentItem()
            {
				IsLayer = false;
            }
			public bool IsLayer; // true if OptionalContent, otherwise is marked content for something else (usually Tagging).
			public int LayerID; // key to layer
			public string LayerName; // actual layer name
        }

		static Stack<MarkedContentItem> markedContent = new Stack<MarkedContentItem>();
		
		static void ProcessElements(Obj ocgs, ElementReader reader)
		{
			Element element;
			while ((element = reader.Next()) != null)
			{
				switch (element.GetType())
				{
               
					case Element.Type.e_marked_content_begin:
					case Element.Type.e_marked_content_point:
						{
							MarkedContentItem mci = new MarkedContentItem();
							Obj tag = element.GetMCTag();
							if(tag != null && tag.IsName())
                            {
								if (tag.GetName().CompareTo("OC") == 0)
								{
									Obj dict = element.GetMCPropertyDict();
									if(dict != null && dict.IsDict())
                                    {
										Obj layerNameObj = dict.FindObj("Name");
										if(layerNameObj != null && layerNameObj.IsString())
                                        {
											mci.LayerName = layerNameObj.GetAsPDFText();
											mci.IsLayer = true;
                                        }
                                    }
								}
							}
							markedContent.Push(mci);
							break;
						}
					case Element.Type.e_marked_content_end:
                        {
							if(markedContent.Count == 0)
                            {
								// ERROR somewhere, maybe malformed PDF content stream.
                            }
							MarkedContentItem item = markedContent.Pop();
							if(item.IsLayer)
                            {
								// OC layer ended
                            }
							break;
                        }
                    case Element.Type.e_image: 				// Process text strings...
						{
							Obj form_obj = element.GetXObject();
							var ocObj = form_obj.FindObj("OC");
							if (ocObj != null && ocObj.IsDict())
                            {
								string imageLayerName = ocObj.FindObj("Name").GetAsPDFText(); // name, not key
								// no push/pop here since an image can have no children.
								// so handle the OC layer at this point.
							}
							break;
						}

					case Element.Type.e_form:				// Process form XObjects
						{
							MarkedContentItem item = null;
							Obj form_obj = element.GetXObject();
							var ocObj = form_obj.FindObj("OC");
							if (ocObj != null && ocObj.IsDict())
							{
								item = new MarkedContentItem();
								item.IsLayer = true;
								item.LayerName = ocObj.FindObj("Name").GetAsPDFText();
								markedContent.Push(item);
								// from ElementReader.FormBegin to ElementReader.End(), everything is under this OC layer
							}
							reader.FormBegin();
							ProcessElements(ocgs, reader);
							reader.End();
							if(item != null)
                            {
								markedContent.Pop();
                            }
							break;
						}
				}
			}
		}

		[STAThread]
		static void Main(string[] args)
		{
            PDFNet.Initialize();

			try
			{
                using (PDFDoc doc = new PDFDoc(path_to_pdf))
				using (ElementReader page_reader = new ElementReader())
				{
					doc.InitSecurityHandler();

					Obj ocgs = doc.GetOCGs();
					if(ocgs != null && !ocgs.IsArray())
                    {
						// ERROR!!!!
						// technically the spec says a Dictionary is allowed, but in practice this is essentially non-existent
                    }

					PageIterator itr;
					for (itr = doc.GetPageIterator(); itr.HasNext(); itr.Next())		//  Read every page
					{
						page_reader.Begin(itr.Current());
						ProcessElements(ocgs, page_reader);
						page_reader.End();
					}
				}

            }
			catch (PDFNetException e)
			{
				Console.WriteLine(e.Message);
			}

		}
	}
}
